<?php
/*	
	("Němečková");
	Michael Finnegan, Monday 5th June 2017	
	
	This program can display a calendar for any year from 1583 to 2099 in a HTML table.
	After the class has been instanciated with the year, call the function displayCalendar.
	Example:

	$Cal = new Calendar(2017);
	$Cal->displayCalendar();

	The matrix array is made public in the object so that the user can create the HTML code to create
	the calendar table of 12 months by examining the values in the matrix.
	The matrix is a 3 dimensional array of months, rows and columns.
	Each month is 0 based. The dates are in their appropriate columns within a grid of 7 columns by 6 rows.
	empty columns in a month grid have their value set to 0. Non-empty columns contain the date (range: 1 to end date of that month)
	in the appropriate column and row.

*/

class Calendar{

	public $sMonthArr = array(	'January', 'February', 'March', 'April', 'May', 'June',
						'July', 'August', 'September', 'October', 'November', 'December');
	private $sDayArr = array('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun');
	public $errNum = 0;
	public $errArr = array();
	private $bigDaysArr = array();			//total # of days in all the previous months of all previous years.
	private $year;
	private $daysInPrevMonths = array();	//total # of days in all the previous months of chosen year.
	private $chosenYearMonths = array();	//The # of days in each month of the chosen year
	public $matrix = array();				//Array of months 0 .. 11 then within each month array matrix of 7 cols by 6 rows.
											//to access a cols use $col = $this->matrix[0][0][0]
	public $HTMLTable = array();				//The HTML table made up in the form of an array of strings.
	/*
		Constructor
		Receive the year. Check if it is valid. If not set error var,
		else set the year.
	*/
	function __construct($year){
		
		$this->year = intval(date('Y'));			//Default to the current year
		if(! is_numeric($year)){
			$this->errNum = 1;
			$this->errArr[] = "Invalid year";

		}else{
			$year = (int) $year;
			if($year < 1583 || $year > 2099){
				$this->errNum =1;
				$this->errArr[] = "Year out of range (1583 - 2099)";
			}else{
				$this->year = $year;
				$this->setupDaysInPrevMonths();
				$this->setupBigDaysArr();
				$this->calendarToMatrix();
				$this->calendarToHTMLTable();		//There are 38 rows in the table including opening & closing table tags.
													//$this->HTMLTable() is the public html table array of 12 months calendar.
			}
		}
	}
	/*-------------------------------------------
		This Easter date calculation function came from 
		https://www.drupal.org/node/1180480
	*/
	public function EasterSunday($year){
		$a = $year % 19;
		$b = intval($year / 100);
		$c = $year % 100;
		$d = intval($b /4);
		$e = $b % 4;
		$f = intval(($b + 8)/25);
		$g = intval(($b - $f + 1)/3);
		$h = (19 * $a + $b - $d - $g + 15) % 30;
		$i = intval($c / 4);
		$k = $c % 4;
		$l = (32 + 2 * $e + 2 * $i - $h - $k) % 7;
		$m = intval(($a + 11 * $h + 22 * $l ) / 451);
		$easterMonth = intval(($h + $l - 7 * $m + 114) / 31);//  [3=March, 4=April]
		$p = ($h + $l - 7* $m + 114) % 31;
		$easterDate = $p + 1; //     (date in Easter Month)

	return array('year' => $year, 'month' => $easterMonth, 'date' => $easterDate);
	}
	/*-------------------------------------------
		Return 0 if not a leap year, else return 1.
	*/
	private function isLeapYear($y){

		$leap =0;
		
		if($y % 400 === 0) {
			$leap = 1;
		}elseif($y % 100 === 0) {
			$leap = 0;
		}elseif( $y % 4 === 0) {
			$leap = 1;
		}
		return $leap;
	}
	//-------------------------------------------
	/*
		Return the total number of days in all the previous years,
		including leap years. 
	*/
	private function daysInPreviousYears(){

		$pY = $this->year -1;
		$leapDays = - intval($pY / 100) + intval($pY / 400) + intval($pY / 4);
		$prevDays = $pY * 365 + $leapDays;		
		return $prevDays;
	}
	//-------------------------------------------
	/*
		Calls member function isLeapYear($this->year);
		Set up an array containing the number of days in each month of the chosen year.
	*/
	private function setupChosenYearMonths(){
		//									0	1	2	3	4	5	6	7	8	9	10	11
		//									Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec
		$this->chosenYearMonths = array(	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
		
		$this->chosenYearMonths[1] = 28 + $this->isLeapYear($this->year);
	}
	//-------------------------------------------
	/*
		Calls member function setupChosenYearMonths();
		Setup the accumulated days in all of the months prior to a given month for the chosen year.
	*/
	private function setupDaysInPrevMonths(){
		
		if($this->errNum){
			return ;
		}
		$this->setupChosenYearMonths();
		$acc = 0;
		for($i = 0; $i < 12; $i += 1) {
			$this->daysInPrevMonths[$i] = $acc;
			$acc += $this->chosenYearMonths[$i];
		}
	}
	/*-------------------------------------------
		Setup the accumulated days in all of the months prior to a given month 
		plus all the years previous to the chosen year.
		function setupDaysInPrevMonths() must have already been called before calling this function.
	*/
	private function setupBigDaysArr(){		
			
		if($this->errNum){
			return ;
		}
		
		$prevDays = $this->daysInPreviousYears();
		for($i = 0; $i < 12; $i += 1) {
			$this->bigDaysArr[$i] = $prevDays + $this->daysInPrevMonths[$i] ;
			
		}

	}
	/*-------------------------------------------
		Receive a day number 1..31 and a month number 1..12
		then return the column 0..6 that this day is on.
		function setupBigDaysArr() must have already been called before calling this function.
		This is done by the constructor.
	*/
	private function getDayCol($d, $m){
		
		$x = ($this->bigDaysArr[$m -1] + $d -1);
		$col = $x % 7;
		
		return $col;
	}
	/*-------------------------------------------
		Receive a day number 1..31 and a month number 1..12
		then return the row 0..5 that this day is on.
		function setupBigDaysArr() must have already been called before calling this function.
		This is done by the constructor.

	*/
	private function getDayRow($d, $m){
		
		$row = intval( ($this->getDayCol(1, $m) + ($d -1)) / 7);
		
		return $row;
	}
	/*-------------------------------------------
		Receive a day number 1..31 and a month number 1..12.
		Return an associative array containing the col (0..6) and
		the row (0..5) that $d, $m is on.
		The days of the week are arranged as
		0   1   2   3   4   5   6
		Mon Tue Wed Thu Fri Sat Sun
	*/
	private function getDayColRow($d, $m){
		
		$colRow = array(
						'col' => $this->getDayCol($d, $m),
						'row' => $this->getDayRow($d, $m)
						);

		return $colRow;
	}
	/*-------------------------------------------
		Initialie the matrix for all of the 12 months. 
		1. Setting the default values to 0.
		2. put the date values in the matrix into their appropriate rows and columns.
	*/
	private function initMatrix(){
		
		for($m = 0; $m < 12; $m +=1) {
			for($r = 0; $r < 6; $r += 1) {
				for($c = 0; $c < 7; $c += 1) {
					$this->matrix[$m][$r][$c] = 0;
				}
			}
		}

	}
	/*-------------------------------------------
		Store a month in the matrix.
		$mm = 1..12
		The matrix must have already been setup with 0 values.
	*/
	private function monthToMatrix($mm){
		
		$m = $mm -1;
		$lastDay = $this->chosenYearMonths[$m];			//$lastDay = total # of days in the month
		for($d = 1; $d <= $lastDay; $d += 1){
			$colRow = $this->getDayColRow($d, $mm);
			$c = $colRow['col'];
			$r = $colRow['row'];
			$this->matrix[$m][$r][$c] = $d;
		}
		
		return;
	}
	/*-------------------------------------------
		Store all the dates for each of the 12 months in the matrix.
	*/
	private function calendarToMatrix(){

		$this->initMatrix();
		for($m = 1; $m <= 12; $m +=1 ){
			$this->monthToMatrix($m);
		}
		return;
	}
	/*-------------------------------------------
		For debugging only.
		See if the values are calculated correctly
		and stored correctly in the arrays by outputting them to the screen. 
		
	*/
	public function testArrays($n = 1){

		echo "<pre>";
		switch($n){
			
			case 1:
				echo "chosenYearMonths<br>";
				print_r($this->chosenYearMonths);
				echo "-----------------------------<br>";
				break;
			
			case 2:
				echo "daysInPrevMonths<br>";
				print_r($this->daysInPrevMonths);
				echo "-----------------------------<br>";
				break;
			
			case 3:
				echo "chosenYearMonths<br>";
				print_r($this->chosenYearMonths);
				echo "-----------------------------<br>";
				echo "daysInPrevMonths<br>";
				print_r($this->daysInPrevMonths);
				echo "-----------------------------<br>";
				break;

			case 4:
				echo "bigDaysArr<br>";
				print_r($this->bigDaysArr);
				echo "-----------------------------<br>";

			case 5:			//output matrix
				echo "matrix<br>";
				print_r($this->matrix);
				echo "-----------------------------<br>";

			
		}
		echo "</pre><br>";
	}
	/*-------------------------------------------
		Receive a month (0..11) and a row (0..5) represents a row of dates to fetch for that month.
		Return a string with the values formatted in HTML table cell style.
	*/
	private function monthRowDatesToHTML($m, $r){
		
		$s = '';
		for($c = 0; $c < 7; $c += 1){
			$d = $this->matrix[$m][$r][$c];
			if($d > 0){
				$s .= '<td class="date">'.$d.'</td>';
			}else{
				$s .= '<td>&nbsp;</td>';
			}
		}

		return $s;
	}
	/*-------------------------------------------
		Put the days of the week into table cells.
	*/
	private function daysOfTheWeekToHTML(){
		$s = "";
		for($i = 0; $i < 7; $i += 1){
			$s .= '<td class="day">'.$this->sDayArr[$i].'</td>';
		}

		return $s;
	}
	/*------------------------------------------
		Based on the value of $row create and return an appropriate string
		containg a calendar row of HTML table cells.
	*/
	private function createHTMLCalendarRow($row){
		
		$s = "";
		$r = $row % 9;
		$lm = 3 * intval($row / 9);
		$leftMonth = $this->sMonthArr[$lm] . " " .$this->year;
		$middleMonth = $this->sMonthArr[$lm + 1]. " ".$this->year;
		$rightMonth = $this->sMonthArr[$lm + 2]. " ".$this->year;
		switch($r){

			case 0: // Month name on each side
				$s = '<tr><td colspan="7" class="m_title">'.$leftMonth.'</td><td>&nbsp;</td><td colspan="7" class="m_title">'.$middleMonth.'</td><td>&nbsp;</td><td colspan="7" class="m_title">'.$rightMonth.'</td></tr>';
				break;

			case 1:	// Days of the week on each side: left, middle, right.
				$s = '<tr>'. $this->daysOfTheWeekToHTML() .'<td>&nbsp;</td>'.$this->daysOfTheWeekToHTML().'<td>&nbsp;</td>';
				$s .= $this->daysOfTheWeekToHTML().'</tr>';
				
				break;
			
			case 8: // A clear line below the month dates.
				$s = '<td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>';
				$s = $s .'<td>&nbsp;</td>'. $s . '<td>&nbsp;</td>'.$s;
				$s = '<tr>'.$s .'</tr>';
				break;

			default: //Assume its one of the six date rows they occur at row positions 2..7 in the HTML calendar
				$datesRow = $r -2;
				$s = '<tr>' . $this->monthRowDatesToHTML($lm, $datesRow) . '<td>&nbsp;</td>' . $this->monthRowDatesToHTML($lm + 1, $datesRow) . '<td>&nbsp;</td>' . $this->monthRowDatesToHTML($lm + 2, $datesRow) .'</tr>';
				break;
		}

		return $s;
	}
	
	
	/*------------------------------------------
		A calendar of 12 months date values has already been setup in the 3 dimensional matrix() array.
		Create a HTML calendar based on the data in matrix array and store the HTML calendar
		as 36 rows of strings in the HTMLTable array.

		Months are displayed in triples i.e., January & February, March, then April, May, June below them.
		Each month has a pattern of 9 rows. The pattern repeats in the months below.
		The 9 rows are:
		0		The month title
		1		The days of the week
		2..7	Date rows 0..5
		8		A blank row before the next pair of two months below.
		Therefore there are 4 * 9 = 36 rows + 2 more rows = 38 (the opening table tag and closing table tag).

	*/
	private function calendarToHTMLTable(){
		
		$this->HTMLTable[0] = '<table class="c_table">';
		
		for($i = 1; $i < 37; $i += 1){
			$this->HTMLTable[$i] = $this->createHTMLCalendarRow($i -1);
		}		
		
		$this->HTMLTable[37] = '</table>';
	
	}
	/*-------------------------------------------
		Show the calendar in the HTML table.

		$this->HTMLTable is exposed to the user of the Calendar object 
		so that he/she can add CSS classes i.D.s to the table to style it nicely.
	*/
	public function displayCalendar(){

		for($i = 0; $i < 38; $i += 1){
			echo $this->HTMLTable[$i];
		}

		return;
	}
	//-------------------------------------------
}




?>
